@@ -29,6 +29,7 @@
 #endif /* !MAC */
 
 ZXchar	AbortChar = CTL('G');	/* VAR: cancels command input */
+bool	DispDefFs = YES;	/* VAR: display default filenames in prompt? */
 
 bool	Asking = NO;
 int	AskingWidth;
@@ -349,8 +350,8 @@ EVexpand()
 
 private char	*fc_filebase;
 bool	DispBadFs = YES;	/* VAR: display filenames with bad extensions? */
-
 char	BadExtensions[sizeof(BadExtensions)] =	/* VAR: extensions to ignore */
+
 # ifndef MSFILESYSTEM
 /* .o: object code format
  * .Z .gz: compressed formats
@@ -456,9 +457,9 @@ int	n;
 				minmatch = numfound == 0
 					? (int)strlen(dir_vec[i])
 #ifdef FILENAME_CASEINSENSITIVE
-					: min(minmatch, numcompcase(dir_vec[lastmatch], dir_vec[i]));
+					: jmin(minmatch, numcompcase(dir_vec[lastmatch], dir_vec[i]));
 #else
-					: min(minmatch, numcomp(dir_vec[lastmatch], dir_vec[i]));
+					: jmin(minmatch, numcomp(dir_vec[lastmatch], dir_vec[i]));
 #endif
 				lastmatch = i;
 				numfound += 1;
@@ -559,7 +560,7 @@ ZXchar	c;
 	} else {
 		/* we're a '?' */
 		const int
-			towidth = min((CO - 2), MAX_TYPEOUT);
+			towidth = jmin((CO - 2), MAX_TYPEOUT);
 		int
 			i,
 			maxlen = 0,
@@ -572,7 +573,7 @@ ZXchar	c;
 		Typeout("Possible completions (in %s):", dir);
 
 		for (i = 0; i < nentries; i++)
-			maxlen = max((int)strlen(dir_vec[i]), maxlen);
+			maxlen = jmax((int)strlen(dir_vec[i]), maxlen);
 		maxlen += 4;	/* pad each column with at least 4 spaces */
 		if (maxlen > towidth)
 		    maxlen = towidth;
@@ -604,7 +605,6 @@ ZXchar	c;
 	freedir(&dir_vec, nentries);
 	return YES;
 }
-
 #endif /* F_COMPLETION */
 
 /* ask for file or directory
@@ -639,7 +639,7 @@ char
 		swritef(prompt, sizeof(prompt), ProcFmt);
 	else
 		jamstr(prompt, prmt);
-	if (def != NULL && *def != '\0') {
+	if (DispDefFs && def != NULL && *def != '\0') {
 		size_t	pl = strlen(prompt);
 
 		swritef(prompt + pl, sizeof(prompt)-pl, "(default %s) ", pretty_name);
