@@ -31,7 +31,7 @@
 #include "marks.h"
 #include "misc.h"
 #include "move.h"
-#include "paragraph.h"
+#include "para.h"
 #include "proc.h"
 #include "reapp.h"
 #include "wind.h"
@@ -116,7 +116,7 @@ const char	*prompt;
 			register const char	**strs = strings;
 			register const struct cmd	*c = commands;
 
-			do ; while ((*strs++ = (*c++).Name) != NULL);
+			do {} while ((*strs++ = (*c++).Name) != NULL);
 		}
 		last = complete(strings, last >= 0? strings[last] : (char *)NULL,
 			prompt, CASEIND | ALLOW_OLD);
@@ -126,7 +126,7 @@ const char	*prompt;
 
 const struct cmd *
 FindCmd(proc)
-register void	(*proc) ptrproto((void));
+register cmdproc_t proc;
 {
 	register const struct cmd	*cp;
 
@@ -156,7 +156,7 @@ register const data_obj	*cp;
 
 			if (cmd->c_proc != NULL) {
 				if ((cmd->Type & MODIFIER)
-				&& BufMinorMode(curbuf, ReadOnly))
+				&& BufMinorMode(curbuf, BReadOnly))
 				{
 					rbell();
 					message("[Buffer is read-only]");
