@@ -63,7 +63,7 @@ int	char1,
 		genbuf[char2] = '\0';
 
 	retline->l_prev = NULL;
-	retline->l_dline = putline(&genbuf[char1]);
+	retline->l_dline = jputline(&genbuf[char1]);
 	patchup(line1, char1, line2, char2);
 
 	if (line1 == line2)
@@ -72,7 +72,7 @@ int	char1,
 		retline->l_next = line1->l_next;
 		(void) ltobuf(line2, genbuf);
 		genbuf[char2] = '\0';
-		line2->l_dline = putline(genbuf);
+		line2->l_dline = jputline(genbuf);
 		/* Shorten this line */
 	}
 
@@ -125,7 +125,7 @@ DelPChar()
 		 * but control characters displayed as two are not handled.
 		 */
 		int	rightcol = calc_pos(linebuf, curchar);
-		int	charcount = min(arg_value(), curchar);
+		int	charcount = jmin(arg_value(), curchar);
 		int	colcount = rightcol - calc_pos(linebuf, curchar-charcount);
 
 		b_char(charcount);
@@ -266,6 +266,18 @@ DelReg()
 	reg_kill(mp->m_line, mp->m_char, NO);
 }
 
+/* get a new line buffer and add it to the kill ring */
+LinePtr
+new_kill()
+{
+	register LinePtr	nl = nbufline();
+
+	AddKillRing(nl);
+	SavLine(nl, NullStr);
+	nl->l_next = nl->l_prev = NULL;
+	return nl;
+}
+    
 /* Save a region.  A pretend kill. */
 
 void
@@ -279,10 +291,7 @@ CopyRegion()
 	if (mp->m_line == curline && mp->m_char == curchar)
 		complain((char *)NULL);
 
-	nl = nbufline();
-	AddKillRing(nl);
-	SavLine(nl, NullStr);
-	nl->l_next = nl->l_prev = NULL;
+	nl = new_kill();
 
 	status = inorder(mp->m_line, mp->m_char, curline, curchar);
 	if (status == -1)
@@ -308,11 +317,11 @@ DelWtSpace()
 		sp -= 1;
 	if (sp != ep) {
 		curchar = sp - linebuf;
-		DFixMarks(curline, curchar, curline, curchar + (ep - sp));
+		DFixMarks(curline, curchar, curline, curchar + (int)(ep - sp));
 		/* Shift the remaining characters left in the buffer to close the gap.
 		 * strcpy(sp, ep) won't do because the destination overlaps the source.
 		 */
-		do ; while ((*sp++ = *ep++) != '\0');
+		do {} while ((*sp++ = *ep++) != '\0');
 		makedirty(curline);
 		modify();
 	}
