@@ -22,8 +22,6 @@
 #	endif /* !MSFILESYSTEM */
 #endif /* !MAC */
 
-#include <errno.h>
-
 #ifdef RAINBOW
 private int rbwrite proto((int, char *, int));
 #endif
@@ -88,19 +86,19 @@ int	flags,
 
 	switch (F_MODE(flags)) {
 	case F_READ:
-		fd = open(name, O_RDONLY | O_BINARY);
+		fd = open(name, O_RDONLY | O_BINARY | O_CLOEXEC);
 		break;
 
 	case F_APPEND:
-		fd = open(name, O_WRONLY | O_BINARY);
+		fd = open(name, O_WRONLY | O_BINARY | O_CLOEXEC);
 		if (fd != -1) {
-			(void) lseek(fd, 0L, 2);
+			(void) lseek(fd, (off_t)0, 2);
 			break;
 		}
 		/* FALLTHROUGH */
 	case F_WRITE:
 #ifdef O_CREAT
-		fd = open(name, O_CREAT | O_TRUNC | O_BINARY | O_RDWR,
+		fd = open(name, O_CREAT | O_TRUNC | O_BINARY | O_RDWR | O_CLOEXEC,
 # ifdef UNIX
 			(jmode_t)CreatMode
 # else
@@ -109,6 +107,21 @@ int	flags,
 			);
 #else
 		fd = creat(name, (jmode_t)CreatMode);
+#endif
+#ifdef O_TRUNC_BROKEN
+		/*
+		 * Cygwin 3.1.2 open(..., O_TRUNC) seems to
+		 * not truncate files on VirtualBox shared
+		 * folders, perhaps on any network drive? But
+		 * it has ftruncate and that appears to
+		 * workaround the problem. Since most uses of
+		 * this function will subsequently write to
+		 * the minibuf, seems pointless to report
+		 * an error on ftruncate, which really should
+		 * never happen if we successfully opened the file!
+		 */
+		if (fd >= 0)
+			(void) ftruncate(fd, 0);
 #endif
 		break;
 
@@ -222,7 +235,7 @@ off_t	offset;
 		flushout(fp);
 	fp->f_cnt = 0;		/* next read will f_filbuf(), next write
 				   will flush() with no bad effects */
-	lseek(fp->f_fd, (long) offset, L_SET);
+	lseek(fp->f_fd, offset, L_SET);
 }
 
 void
