@@ -20,7 +20,7 @@
 #include "marks.h"
 #include "misc.h"
 #include "move.h"
-#include "paragraph.h"
+#include "para.h"
 #include "screen.h"	/* for tabstop */
 #include "sysprocs.h"
 #include "proc.h"
@@ -101,7 +101,7 @@ register int	num;
 
 /* Inserts tabs and spaces to move the cursor to column GOAL.  It
  * Uses the most optimal number of tabs and spaces no matter what
- * was there before hand.
+ * was there beforehand.
  */
 void
 n_indent(goal)
@@ -149,7 +149,7 @@ char	c;
 
 void
 overwrite(c, n)
-char	c;
+DAPchar	c;
 int	n;
 {
 	register int	i;
@@ -205,7 +205,7 @@ SelfInsert()
 /* insert character C N times at point */
 void
 insert_c(c, n)
-char	c;
+DAPchar	c;
 int	n;
 {
 	if (n > 0) {
@@ -491,7 +491,7 @@ Buffer	*whatbuf;
 		buf[tchar] = '\0';
 
 	linecopy(genbuf, atchar, &buf[fchar]);
-	atline->l_dline = putline(genbuf);
+	atline->l_dline = jputline(genbuf);
 	makedirty(atline);
 
 	fline = fline->l_next;
@@ -504,10 +504,10 @@ Buffer	*whatbuf;
 		atchar = 0;
 	}
 
-	getline(atline->l_dline, genbuf);
+	jgetline(atline->l_dline, genbuf);
 	atchar += tchar;
 	linecopy(genbuf, atchar, save);
-	atline->l_dline = putline(genbuf);
+	atline->l_dline = jputline(genbuf);
 	makedirty(atline);
 	IFixMarks(startline, startchar, atline, atchar);
 	bp.p_line = atline;
@@ -568,7 +568,7 @@ YankPop()
 # ifdef __BORLANDC__
 #  include <alloc.h>	/* Borland farmalloc() */
 # else
-#  ifdef __WATCOMC__
+#  ifdef OWCDOS
 #   include <malloc.h>
 #   define farmalloc(sz)	_fmalloc(sz)
 #   define farfree(x)	_ffree(x)
