@@ -46,8 +46,6 @@ extern int UNMACRO(rename)(const char *old, const char *new);	/* <stdin.h> */
 # endif
 #endif /* MSFILESYSTEM */
 
-#include <errno.h>
-
 private void
 	filemunge proto((char *newname)),
 	chk_divergence proto((Buffer *thisbuf, const char *fname, const char *how));
@@ -268,8 +266,8 @@ const char	*complaint;
 # endif
 		) {
 # ifdef O_EXCL
-			int fd = open(buf, O_CREAT | O_EXCL | O_RDWR | O_BINARY
-				, S_IWRITE | S_IREAD);
+			int fd = open(buf, O_CREAT | O_EXCL | O_RDWR | O_BINARY,
+					S_IWRITE | S_IREAD);
 # else /* !O_EXCL */
 			int fd = creat(buf, 0600);
 # endif /* !O_EXCL */
@@ -432,7 +430,7 @@ bool	is_insert;
 	if (!is_insert) {
 		(void) do_stat(curbuf->b_fname, curbuf, DS_SET);
 		set_arg_value((fp->f_flags & F_READONLY)? 1 : 0);
-		TogMinor(ReadOnly);
+		TogMinor(BReadOnly);
 	}
 
 	DOTsave(&save);
@@ -755,7 +753,7 @@ register char	*user,
 		complain((char *)NULL);
 		/* NOTREACHED */
 	}
-	jamstrsub(buf, p->pw_dir, FILESIZE);
+	jamstrsub(buf, p->pw_dir, (size_t)FILESIZE);
 }
 
 # else /* ! USE_GETPWNAM */
@@ -920,7 +918,7 @@ char	*intobuf;
 				int	linklen;
 #endif
 
-				do ; while (dp > intobuf+1 && *--dp != '/');
+				do {} while (dp > intobuf+1 && *--dp != '/');
 #ifdef HAS_SYMLINKS
 				/* If we find that the path up to the .. is a symlink,
 				 * and we don't appear to be in a symlink loop
@@ -1356,9 +1354,9 @@ int	Jr_Len;		/* length of Just Read Line */
 
 void
 #ifdef USE_PROTOTYPES
-getline proto((daddr addr, register char *buf))
+jgetline proto((daddr addr, register char *buf))
 #else
-getline(addr, buf)
+jgetline(addr, buf)
 daddr	addr;
 register char	*buf;
 #endif
@@ -1368,14 +1366,14 @@ register char	*buf;
 
 	lp = buf;
 	bp = getblock(addr, NO);
-	do ; while ((*lp++ = *bp++) != '\0');
+	do {} while ((*lp++ = *bp++) != '\0');
 	Jr_Len = (lp - buf) - 1;
 }
 
 /* Put `buf' and return the disk address */
 
 daddr
-putline(buf)
+jputline(buf)
 char	*buf;
 {
 	register char	*bp,
@@ -1519,7 +1517,7 @@ private daddr	next_bno = 0;
 
 private void	(*blkio) ptrproto((Block *, SSIZE_T (*) ptrproto((int, UnivPtr, size_t))));
 
-/* Needed to comfort dumb MS Visual C */
+/* Needed to comfort MS Visual C */
 private void real_blkio ptrproto((Block *, SSIZE_T (*) ptrproto((int, UnivPtr, size_t))));
 
 private void
@@ -1533,7 +1531,7 @@ register SSIZE_T	(*iofcn) ptrproto((int, UnivPtr, size_t));
 			(iofcn == read) ? "READ" : "WRITE");
 }
 
-/* Needed to comfort dumb MS Visual C */
+/* Needed to comfort MS Visual C */
 private void fake_blkio ptrproto((Block *, SSIZE_T (*) ptrproto((int, UnivPtr, size_t))));
 
 private void
@@ -1803,7 +1801,7 @@ char *fname;
 			fname+dirlen);
 	}
 
-	if ((ffd = open(fname, O_RDONLY | O_BINARY)) < 0)
+	if ((ffd = open(fname, O_RDONLY | O_BINARY | O_CLOEXEC)) < 0)
 		return;	/* cannot open original file: nothing to backup, we assume */
 
 	/* create backup file with same mode as input file */
@@ -1824,7 +1822,7 @@ char *fname;
 		 */
 		if ((unlink(bfname) < 0 && errno != ENOENT)
 #  ifdef O_EXCL
-		|| (bffd = open(bfname, O_CREAT | O_EXCL | O_RDWR, mode)) < 0
+		|| (bffd = open(bfname, O_CREAT | O_EXCL | O_RDWR | O_CLOEXEC, mode)) < 0
 #  else
 		|| (bffd = creat(bfname, mode)) < 0
 #  endif
