@@ -9,6 +9,8 @@
 
 #ifdef IPROCS	/* the body is the rest of this file */
 
+#define MAX_BUSY_WAIT 3	/* seconds to ignore errors on pty */
+
 #include <signal.h>
 
 #include "re.h"
@@ -34,8 +36,6 @@ extern int	UNMACRO(killpg) proto((int /*pgrp*/, int /*sig*/));
 #define killpg(pid, sig)	kill(-(pid), (sig))
 #endif /* USE_KILLPG */
 
-#include <errno.h>
-
 struct process {
 	Process	p_next;
 #ifdef PIPEPROCS
@@ -47,6 +47,7 @@ struct process {
 	int	p_fd;		/* file descriptor of pty? opened r/w */
 # define	p_portpid	p_pid	/* pid of direct child (the shell) */
 	pid_t	p_pid;		/* pid of child (the shell) */
+	time_t	p_start;	/* time we started this child */
 #endif
 	Buffer	*p_buffer;	/* add output to end of this buffer */
 	char	*p_name;	/* ... */
@@ -125,100 +126,20 @@ va_list	ap;
 
 	argv[i++] = va_arg(ap, char *);
 	argv[i++] = (char *)jbasename(argv[0]);	/* lose const (but it's safe) */
-	do ; while ((argv[i++] = va_arg(ap, char *)) != NULL);
+	do {} while ((argv[i++] = va_arg(ap, char *)) != NULL);
 }
 
-/* environment manipulation (always for process) */
+/* environment manipulation for interactive processes */
 
-extern char **environ;	/* <unistd.h> */
-static int	env_headroom = 0;	/* remaining room in environ array */
-
-/* Put a definition into the environment.
- * Same as putenv(3) in SVID 3, POSIX, and BSD 4.3.
- */
-private void
-jputenv(def)
-const char *def;	/* Note: caller must ensure string persists */
-{
-	static bool env_malloced = NO;	/* should we free it when replacing? */
-	const char
-		**const e = (const char **)environ,	/* to avoid type complaints from gcc */
-		**p,
-		*eq;
-
-	if ((eq = strchr(def, '=')) == NULL)
-		return;
-
-	for (p = e; ; p++) {
-		if (*p == NULL) {
-			if (env_headroom == 0) {
-#				define JENV_INCR	5
-				size_t	sz = ((p-e) + 1) * sizeof(char *);
-				const char	**ne = (const char **)
-					malloc(sz + JENV_INCR*sizeof(char *));
-
-				if (ne == NULL)
-					break;	/* malloc failed: give up -- doesn't matter much */
-
-				byte_copy(environ, ne, sz);
-				p = ne + (p-e);
-				if (env_malloced)
-					free((UnivPtr)environ);
-				env_headroom = JENV_INCR;
-				environ = (char **)ne;
-				env_malloced = YES;
-#				undef JENV_INCR
-			}
-			env_headroom -= 1;
-			*p++ = def;
-			*p = NULL;
-			break;
-		}
-		if (strncmp(*p, def, (size_t) (eq - def + 1)) == 0) {
-			*p = def;
-			break;
-		}
-	}
-}
-
-/* Remove any definitions of name from the environment.
- * Same as 4.3BSD's unsetenv(3).
- */
-private void
-junsetenv(name)
-const char *name;
-{
-	char
-		**p = environ,
-		**q = environ;
-	size_t l = strlen(name);
-
-	for (;;) {
-		char *e = *p++;
-
-		*q = e;
-
-		if (e == NULL)
-			break;
-
-		if (strncmp(e, name, l) == 0 && e[l] == '=') {
-			/* unset this one by not advancing q */
-			env_headroom += 1;
-		} else {
-			q += 1;
-		}
-	}
-}
+private Env iproc_env;
+private char IEnvExpBuf[LBSIZE];
+private char IEnvUnsetBuf[LBSIZE];
 
 /* Erase misleading knowledge of the terminal type from the environment.
  * The value of variable TERMCAP has two interpretations:
  * If it starts with /, it is a path to the database.
  * If it does not, it is the database entry.
- *
- * We also set EMACS=t to mimic GNU EMACS.  The GNU EMACS manual says:
- *	Emacs defines the environment variable `EMACS' in the subshell,
- *	with value `t'.  A shell script can check this variable to
- *	determine whether it has been run from an Emacs subshell.
+ * Only happens in child.
  */
 private void
 set_process_env()
@@ -227,13 +148,33 @@ set_process_env()
 	const char *tc = getenv(tcn);
 
 	if (tc != NULL && tc[0] != '/')
-		junsetenv(tcn);
-#ifndef IPROC_TERM
-	junsetenv("TERM");
-#else
-	jputenv(IPROC_TERM);
-#endif
-	jputenv("EMACS=t");	
+		junsetenv(&iproc_env, tcn);
+	environ = (char **) jenvdata(&iproc_env); /* avoid gcc warning */
+}
+
+void
+IprocEnvExport()
+{
+	jamstr(IEnvExpBuf, ask(IEnvExpBuf, ProcFmt));
+	jputenv(&iproc_env, IEnvExpBuf);
+}
+
+void
+IprocEnvShow()
+{
+	const char **p;
+	TOstart("i-shell environment");
+	for (p = jenvdata(&iproc_env); *p; p++) {
+	    Typeout("%s", *p);
+	}
+	TOstop();
+}
+
+void
+IprocEnvUnset()
+{
+	jamstr(IEnvUnsetBuf, ask(IEnvUnsetBuf, ProcFmt));
+	junsetenv(&iproc_env, IEnvUnsetBuf);
 }
 
 /* There are two very different implementation techniques: pipes and ptys.
@@ -241,11 +182,12 @@ set_process_env()
  * the two techniques: the first uses pipes and the second uses ptys.
  */
 
+
+#include "ttystate.h"
 #ifdef PIPEPROCS
 
-#include <sgtty.h>
-
 char	Portsrv[FILESIZE];	/* path to portsrv program (in LibDir) */
+const char	*procarg0 = "jove-portsrv"; /* helpful identifying it in ps */
 
 int	NumProcs = 0;
 
@@ -261,8 +203,9 @@ register int	nbytes;
 {
 	register Process	p = proc_pid(pid);
 
+	jdbg("read_pipe_proc pid %D %d\n", (long) pid, nbytes);
 	if (p == NULL) {
-		writef("\riproc: unknown pid (%d)", (int)pid);
+		writef("\riproc: unknown pid (%D)", (long) pid);
 	} else if (p->p_io_state == IO_NEW) {
 		/* first message: pid of real child, not of portsrv */
 		pid_t	rpid;
@@ -300,7 +243,7 @@ register int	nbytes;
 		while (nbytes > 0) {
 			char	ibuf[512+1];	/* NOTE: room for added NUL */
 			size_t n = f_readn(ProcInput, ibuf,
-				(size_t)min((int)(sizeof ibuf) - 1, nbytes));
+				(size_t)jmin((int)(sizeof ibuf) - 1, nbytes));
 
 			nbytes -= n;
 			proc_rec(p, ibuf, n);
@@ -325,6 +268,8 @@ proc_close(p)
 Process	p;
 {
 	if (p->p_toproc >= 0) {
+		jdbg("closing proc fd %d pid %D NumProcs %d\n",
+		     p->p_toproc, (long) (p->p_pid), NumProcs);
 		(void) close(p->p_toproc);
 		p->p_toproc = -1;	/* writes will fail */
 		NumProcs -= 1;
@@ -377,13 +322,15 @@ proc_strt(bufname, clobber, procname, va_alist)
 	untieDeadProcess(buf_exists(bufname));
 	isprocbuf(bufname);	/* make sure BUFNAME is either nonexistant
 				   or is of type B_PROCESS */
-	if (access(Portsrv, X_OK) < 0) {
+	if (access(Portsrv, J_X_OK) < 0) {
 		complain("[Couldn't access %s: %s]", Portsrv, strerror(errno));
 		/* NOTREACHED */
 	}
 
 	dopipe(toproc);
 
+	jdbg("proc_strt NumProcs=%d buf \"%s\" proc \"%s\"\n",
+	     NumProcs, bufname, procname);
 	if (NumProcs++ == 0)
 		kbd_strt();	/* may create kbd process: must be done before fork */
 	switch (pid = fork()) {
@@ -395,7 +342,7 @@ proc_strt(bufname, clobber, procname, va_alist)
 		/* NOTREACHED */
 
 	case 0:
-		argv[0] = "portsrv";
+		argv[0] = (char*)procarg0;
 		va_init(ap, procname);
 		make_argv(&argv[1], ap);
 		va_end(ap);
@@ -405,6 +352,8 @@ proc_strt(bufname, clobber, procname, va_alist)
 		pipeclose(toproc);
 		jcloseall();
 		set_process_env();
+		jdbg("ready to exec %s %s %s %s\n", Portsrv, argv[0],
+		     argv[1]? argv[1] : "(null)", argv[2]? argv[2] : "(null)");
 		execv(Portsrv, argv);
 		raw_complain("execl failed: %s", strerror(errno));
 		_exit(1);
@@ -460,7 +409,10 @@ kbd_init()
 	 */
 	int	p[2];
 
-	(void) pipe(p);
+	if (pipe(p) < 0) {
+	    complain("Cannot create pipe to kbd process! %s\n", strerror(errno));
+	    /* NOTREACHED */
+	}
 	ProcInput = fd_open("process-input", F_READ|F_LOCKED, p[0],
 			    (char *)NULL, 512);
 	ProcOutput = p[1];
@@ -473,9 +425,13 @@ kbd_init()
 		(void) setsighandler(SIGINT, SIG_IGN);
 		(void) setsighandler(SIGALRM, SIG_IGN);
 		close(1);
-		dup(ProcOutput);
+		if (dup(ProcOutput) < 0) {
+		    raw_complain("Cannot dup output fd: %s", strerror(errno));
+		    exit(-1);
+		}
 		jcloseall();
-		execl(Portsrv, "kbd", (char *)NULL);
+		jdbg("ready to exec %s as %s --kbd\n", Portsrv, procarg0);
+		execl(Portsrv, procarg0, "--kbd", (char *)NULL);
 		raw_complain("kbd exec failed: %s", strerror(errno));
 		exit(-1);
 	}
@@ -490,6 +446,8 @@ private bool	kbd_state = NO;
 void
 kbd_strt()
 {
+	jdbg("kbd_strt state %d pid %D NumProcs %d\n",
+	     kbd_state, (long) kbd_pid, NumProcs);
 	if (!kbd_state) {
 		if (kbd_pid == -1)
 			kbd_init();
@@ -502,6 +460,8 @@ kbd_strt()
 bool
 kbd_stop()
 {
+	jdbg("kbd_stop state %d pid %D NumProcs %d\n",
+	     kbd_state, (long) kbd_pid, NumProcs);
 	if (kbd_state) {
 		kbd_state = NO;
 		kill(kbd_pid, KBDSIG);
@@ -513,6 +473,8 @@ kbd_stop()
 void
 kbd_kill()
 {
+	jdbg("kbd_kill state %d pid %D Numprocs %d\n",
+	     kbd_state, (long) kbd_pid, NumProcs);
 	if (kbd_pid != -1) {
 		kill(kbd_pid, SIGKILL);
 		kbd_pid = -1;
@@ -524,13 +486,15 @@ kbd_kill()
 #include <sys/time.h>
 #include "select.h"
 
-#include "ttystate.h"
-
 # ifdef USE_OPENPTY	/* modern BSDs have openpty(3) */
 #  ifdef HAVE_LIBUTIL_H	/* but disagree about header! */
 #   include <libutil.h>
 #  else
-#   include <util.h>
+#   ifdef HAVE_PTY_H
+#    include <pty.h>
+#   else
+#    include <util.h>
+#   endif
 #  endif
 # endif
 
@@ -538,7 +502,7 @@ kbd_kill()
 #  include <stdlib.h>	/* for grantpt and unlockpt, at least in Solaris 2.3 */
 #  if _XOPEN_SOURCE >= 500
     /* Linux/glibc no longer pretends to support STREAMS (XSR) (2008) */
-#   if _XOPEN_STREAMS != -1
+#   if defined(_XOPEN_STREAMS) && _XOPEN_STREAMS != -1
 #    include <stropts.h>
 #   endif
 #  else
@@ -570,29 +534,33 @@ register int	fd;
 	}
 
 	n = read(fd, (UnivPtr) ibuf, sizeof(ibuf) - 1);
+	jdbg("pty read %d returned %d errno %d from %s\n", fd, n, errno, 
+	     p->p_name);
 	if (n <= 0) {
 		if (n < 0) {
-			switch (errno) {
-			case EIO:
-			case EWOULDBLOCK:
-#if EWOULDBLOCK != EAGAIN
-			case EAGAIN:
-#endif
-				/* ??? On some systems, pty reads fail initially, for
-				 * reasons that are not clear to me (DHR).  This code
-				 * forgives these specific kinds of failure until the
-				 * process leaves the NEW state.  We hope that this
-				 * does not cause a long busy wait.
+			if (errno == EIO || RETRY_ERRNO(errno)) {
+				/*
+				 * ??? On some systems, pty reads fail
+				 * initially, for reasons that are not clear to
+				 * me (DHR).  This code forgives these specific
+				 * kinds of failure until the process leaves the
+				 * NEW state.  We hope that this does not cause
+				 * a long busy wait.
 				 */
-				if (p->p_io_state == IO_NEW)
-					return;
+				if (p->p_io_state == IO_NEW) {
+					time_t now = time(NULL);
+					jdbg("IO_NEW err %d start %D now %D\n",
+					     errno, (long)p->p_start,(long)now);
+					if (now < p->p_start+MAX_BUSY_WAIT)
+					    return;
+				}
+				jdbg("treating error as EOF\n");
 
 				/* We get here if the i-proc closes stdout
 				 * before exiting (eg. Bourne Shell),
 				 * so we treat it as a simple EOF.
 				 */
-				break;
-			default:
+			} else {
 				/* true I/O error */
 				swritef(ibuf, sizeof(ibuf),
 					"\n[pty read error: %s]\n", strerror(errno));
@@ -615,6 +583,7 @@ ProcCont()
 {
 	Process	p = curbuf->b_process;
 
+	jdbg("ProcCont pid %D %s\n", (long)(p->p_pid), p->p_name);
 	if (proc_kill(p, SIGCONT) && p->p_child_state == C_STOPPED) {
 		p->p_child_state = C_LIVE;
 		UpdModLine = YES;
@@ -710,17 +679,21 @@ int	sig;
 {
 	Process	p;
 
+	jdbg("send_sig %d\n", sig);
 	if ((p = curbuf->b_process) == NULL || p->p_fd < 0)
 		complain("[No process]");
 	ToLast();
 #  ifdef TIOCSIG
+	jdbg("TIOCSIG pid %d %s\n", p->p_pid, p->p_name);
 	if (ioctl(p->p_fd, TIOCSIG, sig) < 0)
 		complain("TIOCSIG failed: %d %s", errno, strerror(errno));
 #  else /* !TIOCSIG */
 #   ifdef SVR4_PTYS
+	jdbg("SVR4 TIOCSIGNAL pid %d %s\n", p->p_pid, p->p_name);
 	if (ioctl(p->p_fd, TIOCSIGNAL, sig) < 0)
 		complain("TIOCSIGNAL failed: %d %s", errno, strerror(errno));
 #   else /* !SVR4_PTYS */
+	jdbg("TIOCSIGNAL pid %d %s\n", p->p_pid, p->p_name);
 	if (ioctl(p->p_fd, TIOCSIGNAL, (void *) &sig) < 0)
 		complain("TIOCSIGNAL failed: %d %s", errno, strerror(errno));
 #   endif /* !SVR4_PTYS */
@@ -745,6 +718,7 @@ char	c;
 
 	if ((p = curbuf->b_process) == NULL || p->p_fd < 0)
 		complain("[No process]");
+	jdbg("send_xc 0x%x pid %d %s \n", c, p->p_pid, p->p_name);
 	ToLast();
 	{
 		char	buf[1+1];	/* NOTE: room for added NUL */
@@ -758,6 +732,7 @@ char	c;
 				off = 0,
 				on = 1;
 
+			jdbg("TIOCREMOTE off pid %d %s\n", p->p_pid, p->p_name);
 			while (ioctl(p->p_fd, TIOCREMOTE, (UnivPtr) &off) < 0)
 				if (errno != EINTR)
 					complain("TIOCREMOTE OFF failed: %d %s", errno, strerror(errno));
@@ -778,6 +753,7 @@ char	c;
 				break;
 			}
 #  ifndef NO_TIOCREMOTE
+			jdbg("TIOCREMOTE on pid %d %s\n", p->p_pid, p->p_name);
 			while (ioctl(p->p_fd, TIOCREMOTE, (UnivPtr) &on) < 0)
 				if (errno != EINTR)
 					complain("TIOCREMOTE ON failed: %d %s", errno, strerror(errno));
@@ -803,6 +779,7 @@ ProcEof()
 		complain("[No process]");
 	ToLast();
 	proc_rec(p, mess, sizeof(mess)-1);
+	jdbg("ProcEof pid %d %s\n", p->p_pid, p->p_name);
 	while (write(p->p_fd, (UnivPtr) mess, (size_t)0) < 0)
 		if (errno != EINTR)
 			complain("[error writing EOF to iproc: %d %s]", errno, strerror(errno));
@@ -846,6 +823,7 @@ private void
 proc_close(p)
 Process p;
 {
+	jdbg("proc_close %d %s\n", p->p_pid, p->p_name);
 	if (p->p_fd >= 0) {
 		(void) close(p->p_fd);
 		FD_CLR(p->p_fd, &global_fd);
@@ -861,6 +839,7 @@ Process p;
 char	*buf;
 size_t	nbytes;
 {
+	jdbg("proc_write %d bytes to pid %d %s\n", nbytes, p->p_pid, p->p_name);
 	if (p->p_fd >= 0) {
 		fd_set	mask;
 
@@ -891,9 +870,7 @@ proc_strt(bufname, clobber, procname, va_alist)
 	Process	newp;
 	Buffer	*newbuf;
 	int	ptyfd = -1;
-# ifdef BSD_PTYS
 	int	slvptyfd = -1;
-# endif
 
 # if !defined(TERMIO) && !defined(TERMIOS)
 #  ifdef TIOCSETD
@@ -923,14 +900,15 @@ proc_strt(bufname, clobber, procname, va_alist)
 #  endif
 # endif
 
+	strcpy(ttybuf, "?");	 /* to safely print when debugging */
 	untieDeadProcess(buf_exists(bufname));
 	isprocbuf(bufname);	/* make sure BUFNAME is either nonexistant
 				   or is of type B_PROCESS */
 	va_init(ap, procname);
 	make_argv(argv, ap);
 	va_end(ap);
-	if (access(argv[0], X_OK) != 0) {
-		complain("[Couldn't access %s: %s]", argv[0], strerror(errno));
+	if (access(argv[0], J_X_OK) != 0) {
+		complain("[Couldn't access shell %s: %s]", argv[0], strerror(errno));
 		/* NOTREACHED */
 	}
 
@@ -945,17 +923,19 @@ proc_strt(bufname, clobber, procname, va_alist)
 		register char	*s = _getpty(&ptyfd, O_RDWR | O_NDELAY, 0600, 0);
 
 		if (s == NULL) {
-			message("[No ptys!]");
+			message("[No pty from getpty!]");
 			goto fail;
 		}
 		(void)strcpy(ttybuf, s);
+		jdbg("IRIX_PTYS _getpty ptyfd %d %s\n", ptyfd, ttybuf);
 	}
 # endif /* IRIX_PTYS */
 # ifdef SVR4_PTYS
 	if ((ptyfd = open("/dev/ptmx", O_RDWR | O_BINARY)) < 0) {
-		message("[No ptys!]");
+		message("[No pty from /dev/ptmx!]");
 		goto fail;
 	}
+	jdbg("SVR4_PTYS ptmx ptyfd %d\n", ptyfd);
 #  ifndef GRANTPT_BUG
 	/* grantpt() seems to be implemented using a fork/exec.
 	 * This is done to allow grantpt do do priviledged things
@@ -972,7 +952,17 @@ proc_strt(bufname, clobber, procname, va_alist)
 		message("[unlockpt failed]");
 		goto fail;
 	}
+	jdbg("SVR4_PTYS grantpt and unlockpt ptyfd %d\n", ptyfd);
 #  endif /* !GRANTPT_BUG */
+#ifdef TIOCGPTPEER
+	/* if we have TIOCGPTPEER (Linux post-2017), much better to use it! */
+	slvptyfd = ioctl(ptyfd, TIOCGPTPEER, O_RDWR | O_NOCTTY);
+	if (slvptyfd == -1 && errno != EINVAL && errno != ENOTTY) {
+		message("[ioctl TIOCGPTPEER failed]");
+		goto fail;
+	}
+	jdbg("SVR4_PTYS TIOCGPTPEER ptyfd %d slv %d\n", ptyfd, slvptyfd);
+#endif
 	{
 		register char	*s = ptsname(ptyfd);
 
@@ -981,8 +971,10 @@ proc_strt(bufname, clobber, procname, va_alist)
 			goto fail;
 		}
 		strcpy(ttybuf, s);
+		jdbg("SVR4_PTYS ptyfd %d %s\n", ptyfd, ttybuf);
 	}
 #  ifdef TIOCFLUSH
+	jdbg("TIOCFLUSH ptyfd %d\n", ptyfd);
 	(void) ioctl(ptyfd, TIOCFLUSH, (UnivPtr) NULL);	/* ??? why? */
 #  endif
 # endif /* SVR4_PTYS */
@@ -990,9 +982,10 @@ proc_strt(bufname, clobber, procname, va_alist)
 #  ifdef USE_OPENPTY
 	if (openpty(&ptyfd, &slvptyfd, ttybuf, NULL, NULL) < 0)
 	{
-		message("[Out of ptys!]");
+		message("[No pty from openpty!]");
 		goto fail;
 	}
+	jdbg("openpty ptyfd %d slv %d %s\n", ptyfd, slvptyfd, ttybuf);
 #  else /* !USE_OPENPTY */
 	{
 		register const char	*s;
@@ -1001,11 +994,12 @@ proc_strt(bufname, clobber, procname, va_alist)
 			register const char	*t;
 
 			if (*s == '\0') {
-				message("[Out of ptys!]");
+				message("[Out of ptys in /dev/pty*!]");
 				goto fail;
 			}
 			for (t = "0123456789abcdef"; *t; t++) {
 				swritef(ttybuf, sizeof(ttybuf), "/dev/pty%c%c", *s, *t);
+				jdbg("trying pty %s\n", ttybuf);
 				if ((ptyfd = open(ttybuf, O_RDWR | O_BINARY)) >= 0) {
 					ttybuf[5] = 't';	/* pty => tty */
 					/* Make sure other end is available too */
@@ -1031,25 +1025,31 @@ proc_strt(bufname, clobber, procname, va_alist)
 		s_mess("[Couldn't access %s: %s]", ttybuf, strerror(errno));
 		goto fail;
 	}
+	jdbg("can access pty %s\n", ttybuf);
 # endif /* !GRANTPT_BUG */
 
 # if !defined(TERMIO) && !defined(TERMIOS)
 #  ifdef TIOCGETD
+	jdbg("TIOCGETD %s\n", ttybuf);
 	(void) ioctl(0, TIOCGETD, (UnivPtr) &ldisc);
 #  endif
 #  ifdef TIOCLGET
+	jdbg("TIOCLGET %s\n", ttybuf);
 	(void) ioctl(0, TIOCLGET, (UnivPtr) &lmode);
 #  endif
 # endif /* !defined(TERMIO) && !defined(TERMIOS) */
 
 # ifdef TIOCGWINSZ
+	jdbg("TIOCGWINSZ %s\n", ttybuf);
 	(void) ioctl(0, TIOCGWINSZ, (UnivPtr) &win);
 # else
 #  ifdef BTL_BLIT
+	jdbg("JWINSIZE %s\n", ttybuf);
 	(void) ioctl(0, JWINSIZE, (UnivPtr) &jwin);
 #  endif /* BTL_BLIT */
 # endif
 
+	jdbg("before fork, ptyfd %d slv %d buf %s\n", ptyfd, slvptyfd, ttybuf);
 	switch (pid = fork()) {
 	case -1:
 		/* fork failed */
@@ -1080,6 +1080,7 @@ proc_strt(bufname, clobber, procname, va_alist)
 		 * us giving good diagnostics for grantpt and unlockpt.
 		 * I hope I've found all the signals caught everywhere else.
 		 */
+		jdbg("child grantpt bug, ptyfd %d slv %d buf %s\n", ptyfd, slvptyfd, ttybuf);
 		(void) setsighandler(SIGCHLD, SIG_DFL);
 		(void) setsighandler(SIGWINCH, SIG_DFL);
 		(void) setsighandler(SIGALRM, SIG_DFL);
@@ -1105,6 +1106,7 @@ proc_strt(bufname, clobber, procname, va_alist)
 		(void) close(2);
 
 # ifdef TERMIOS
+		jdbg("child setsid %s\n", ttybuf);
 		setsid();
 # else /* !TERMIOS */
 #  ifdef TIOCNOTTY
@@ -1112,6 +1114,7 @@ proc_strt(bufname, clobber, procname, va_alist)
 		{
 			int	i = open("/dev/tty", O_RDWR | O_BINARY);
 
+			jdbg("child TIOCNOTTY %d %s\n", i, ttybuf);
 			if (i >= 0) {
 				(void) ioctl(i, TIOCNOTTY, (UnivPtr)NULL);
 				(void) close(i);
@@ -1119,17 +1122,23 @@ proc_strt(bufname, clobber, procname, va_alist)
 		}
 #  endif /* TIOCNOTTY */
 # endif /* !TERMIOS */
-		if (open(ttybuf, O_RDWR | O_BINARY) != 0)
-			_exit(errno+1);
-		(void) dup2(0, 1);
-		(void) dup2(0, 2);
-
-# ifdef BSD_PTYS
-		close(slvptyfd);	/* safe to close now that std* are open */
-# endif /* BSD_PTYS */
-
+		jdbg("child ptyfd %d slv %d buf %s\n", ptyfd, slvptyfd, ttybuf);
+		if (slvptyfd < 0) {
+			if ((slvptyfd = open(ttybuf, O_RDWR | O_BINARY)) != 0)
+				_exit(errno+1);
+			jdbg("child slv slv %d\n", slvptyfd);
+		} /* else use what we got from openpty */
+		(void) dup2(slvptyfd, 1);
+		(void) dup2(slvptyfd, 2);
+		if (slvptyfd != 0) {
+			(void) dup2(slvptyfd, 0);
+			jdbg("child closing slv %d %s\n", slvptyfd, ttybuf);
+			(void) close(slvptyfd);	/* safe to close now that std* are open */
+		}
+		jdbg("child std 0, 1, 2 duped\n");
 # ifdef SVR4_PTYS
 #  ifdef I_PUSH		/* LINUX/glibc no longer even pretends to support this (2008) */
+		jdbg("child SVR4_PTYS I_PUSH ptem ldterm ttcompat %s\n", ttybuf);
 		(void) ioctl(0, I_PUSH, (UnivPtr) "ptem");
 		(void) ioctl(0, I_PUSH, (UnivPtr) "ldterm");
 		(void) ioctl(0, I_PUSH, (UnivPtr) "ttcompat");
@@ -1140,10 +1149,14 @@ proc_strt(bufname, clobber, procname, va_alist)
 		/* Make this controling tty.
 		 * This is needed by OSF.  It may be needed by BSDPOSIX systems.
 		 * It should not hurt on any system that defines TIOCSCTTY.
+		 * On Solaris (OpenIndiana 2019.11), the ioctl fails with errno 25 (ENOTTY)
+		 * but that seems harmless, so we just log but ignore error returns
+		 * (as used to be the case before 4.16.0.31!)
 		 */
-		if (ioctl(0, TIOCSCTTY) == -1)
-			_exit(errno+1);	/* no good way to signal user */
-
+		jdbg("child TIOCSTTY %s\n", ttybuf);
+		if (ioctl(0, TIOCSCTTY) < 0) {
+			jdbg("TIOCSCTTY failed errno %d %s\n", errno, strerror(errno));
+		}
 # endif
 
 # ifndef NO_TIOCREMOTE
@@ -1159,38 +1172,46 @@ proc_strt(bufname, clobber, procname, va_alist)
 		{
 			int	on = 1;
 
+			jdbg("child TIOCREMOTE on %d %s\n", ptyfd, ttybuf);
 			if (ioctl(ptyfd, TIOCREMOTE, (UnivPtr) &on) < 0)
 				_exit(errno+1);	/* no good way to signal user */
 		}
 # endif
+		jdbg("child closing %d %s\n", ptyfd, ttybuf);
 		close(ptyfd);
 
 # if !defined(TERMIO) && !defined(TERMIOS)
 #  ifdef TIOCSETD
+		jdbg("child TIOCSETD %s\n", ttybuf);
 		(void) ioctl(0, TIOCSETD, (UnivPtr) &ldisc);
 #  endif
 #  ifdef TIOCLSET
+		jdbg("child TIOCLSET %s\n", ttybuf);
 		(void) ioctl(0, TIOCLSET, (UnivPtr) &lmode);
 #  endif
 #  ifdef TIOCSETC
 		(void) ioctl(0, TIOCSETC, (UnivPtr) &tc[NO]);
 #  endif
 #  ifdef USE_TIOCSLTC
+		jdbg("child TIOCSLTC %s\n", ttybuf);
 		(void) ioctl(0, TIOCSLTC, (UnivPtr) &ls[NO]);
 #  endif
 # endif /* !defined(TERMIO) && !defined(TERMIOS) */
 
 # ifdef TIOCGWINSZ
+		jdbg("child TIOCSWINSZ %s\n", ttybuf);
 		win.ws_row = curwind->w_height;
 		(void) ioctl(0, TIOCSWINSZ, (UnivPtr) &win);
 # else /* !TIOCGWINSZ */
 #  ifdef BTL_BLIT
+		jdbg("child JSWINSIZE %s\n", ttybuf);
 		jwin.bytesy = curwind->w_height;
 		(void) ioctl(0, JSWINSIZE, (UnivPtr) &jwin);
 #  endif
 # endif /* !TIOCGWINSZ */
 
 # if defined(TERMIO) || defined(TERMIOS)
+		jdbg("child set termio(s)%s\n", ttybuf);
 		sgt = sg[NO];
 		sgt.c_iflag &= ~(IGNBRK | BRKINT | ISTRIP | INLCR | IGNCR | ICRNL
 #  ifdef IXANY	/* not in QNX */
@@ -1200,46 +1221,59 @@ proc_strt(bufname, clobber, procname, va_alist)
 		sgt.c_lflag &= ~(ECHO);
 		sgt.c_oflag &= ~(ONLCR | TABDLY);
 #  ifdef TERMIO
-		do ; while (ioctl(0, TCSETAW, (UnivPtr) &sgt) < 0 && errno == EINTR);
+		jdbg("child TCSETAW %s\n", ttybuf);
+		do {} while (ioctl(0, TCSETAW, (UnivPtr) &sgt) < 0 && errno == EINTR);
 #  endif
 #  ifdef TERMIOS
-		do ; while (tcsetattr(0, TCSADRAIN, &sgt) < 0 && errno == EINTR);
+		jdbg("child TCSADRAIN %s\n", ttybuf);
+		do {} while (tcsetattr(0, TCSADRAIN, &sgt) < 0 && errno == EINTR);
 #  endif
 # else /* !(defined(TERMIO) || defined(TERMIOS)) */
+		jdbg("child set sgt %s\n", ttybuf);
 		sgt = sg[NO];
 		sgt.sg_flags &= ~(ECHO | CRMOD | ANYP | ALLDELAY | RAW | LCASE | CBREAK | TANDEM);
 		(void) stty(0, &sgt);
 # endif /* !(defined(TERMIO) || defined(TERMIOS)) */
 
+		jdbg("child newpg %s\n", ttybuf);
 		NEWPG();
-# ifdef POSIX_PROCS
-		tcsetpgrp(0, getpid());
-# else /* !POSIX_PROCS */
 		{
 			int	i = getpid();
 
+# ifdef POSIX_PROCS
+			jdbg("child tcsetpgrp %d %s\n", i, ttybuf);
+			tcsetpgrp(0, getpid());
+# else /* !POSIX_PROCS */
+			jdbg("child TIOCSPGRP %d %s\n", i, ttybuf);
 			(void) ioctl(0, TIOCSPGRP, (UnivPtr) &i);
-		}
 # endif /* POSIX_PROCS */
+		}
 
 		set_process_env();
+		jdbg("child ready to exec %s %s\n", argv[0], argv[1]);
 		execvp(argv[0], &argv[1]);
 		raw_complain("execvp failed! %s", strerror(errno));
 		_exit(errno + 1);
 	}
 
 # ifdef BSD_PTYS
+	jdbg("closing slv %d %s\n", slvptyfd, ttybuf);
 	close(slvptyfd);
 # endif /* BSD_PTYS */
 
 	newp = (Process) emalloc(sizeof *newp);
 
 # ifdef O_NDELAY
+	jdbg("O_NDELAY %d %s\n", ptyfd, ttybuf);
 	fcntl(ptyfd, F_SETFL, O_NDELAY);
 # endif
 # ifdef O_NONBLOCK
+	jdbg("O_NONBLOCK %d %s\n", ptyfd, ttybuf);
 	fcntl(ptyfd, F_SETFL, O_NONBLOCK);
 # endif
+	(void) time(&(newp->p_start));
+	jdbg("start pid %d fd %d %s %D\n", pid, ptyfd, ttybuf,
+	     (long) newp->p_start);
 	newp->p_fd = ptyfd;
 	newp->p_pid = pid;
 
@@ -1270,6 +1304,7 @@ proc_strt(bufname, clobber, procname, va_alist)
 	return;
 
 fail:
+	jdbg("pty open fail %d %d %s\n", ptyfd, slvptyfd, ttybuf);
 	if (ptyfd >= 0)
 		close(ptyfd);
 # ifdef BSD_PTYS
@@ -1333,7 +1368,8 @@ closeiprocs()
 
 #endif /* !PIPEPROCS */
 
-char	proc_prompt[128] = "% ";	/* VAR: process prompt */
+/* This is for the shell window. Supports sh, csh and ksh. */
+char	proc_prompt[128] = "^[^%$#]*[%$#] ";	/* VAR: process prompt */
 
 const char *
 pstate(p)
@@ -1701,7 +1737,7 @@ ShellProc()
 	char	shbuf[20];
 	register Buffer	*b;
 
-	swritef(shbuf, sizeof(shbuf), "*shell-%d*", arg_value());
+	swritef(shbuf, sizeof(shbuf), "[shell-%d]", arg_value());
 	b = buf_exists(shbuf);
 	if (b == NULL || dead(b->b_process)) {
 		char	cbnspace[FILESIZE];
