@@ -277,7 +277,7 @@ private const unsigned char IdChartable[NCHARS] = {
 
 bool
 jisident(c)
-char	c;
+DAPchar	c;
 {
 #ifdef USE_CTYPE
 	return (IdChartable[ZXC(c)] & (1 << curbuf->b_major)) != 0 || jisword(c);
