@@ -47,7 +47,6 @@
 #endif
 
 #include <signal.h>
-#include <errno.h>
 
 #ifdef MAC
 # include "mac.h"
@@ -61,12 +60,13 @@
 # include <time.h>
 # include <process.h>
 # define SIGHUP	99
-# if defined(__WATCOMC__)
+# ifdef OWCDOS
 #  include <malloc.h>	/* for _heapgrow */
 # endif
 #endif /* MSDOS */
 
 #ifdef WIN32
+# undef CR /* sigh, used as a field name in some windows header! */
 # include <windows.h>	/* ??? is this needed? */
 # undef FIONREAD	 /* This is defined but ioctl isn't so we cannot use it. */
 #endif
@@ -232,7 +232,9 @@ SIGRESTYPE
 finish(code)
 int	code;
 {
+#ifdef RECOVER
 	int save_errno = errno;	/* Subtle, but necessary! */
+#endif
 	bool	DelTmps = YES;		/* Usually we delete them. */
 
 	DisabledRedisplay = YES;
@@ -303,7 +305,7 @@ int	code;
 
 	resetsighandler(SIGINT, handle_sigint);
 	f_mess("Abort (Type 'n' if you're not sure)? ");
-	Placur(ILI, min(CO - 2, calc_pos(mesgbuf, MAXCOLS)));
+	Placur(ILI, jmin(CO - 2, calc_pos(mesgbuf, MAXCOLS)));
 	flushscreen();
 # ifdef UNIX
 #  ifdef PIPEPROCS
@@ -318,20 +320,9 @@ int	code;
 #  ifdef NONBLOCKINGREAD
 	setblock(YES);	/* turn blocking on (in case it was off) */
 #  endif
-	for (;;) {
+	do {
 		c = 'n';
-		if (read(0, (UnivPtr) &c, sizeof(c)) < 0) {
-			switch (errno) {
-			case EINTR:
-#  if EWOULDBLOCK != EAGAIN	/* aliases in System Vr4 */
-			case EWOULDBLOCK:
-#  endif
-			case EAGAIN:
-				continue;
-			}
-		}
-		break;
-	}
+	} while (read(0, (UnivPtr) &c, sizeof(c)) < 0 && RETRY_ERRNO(errno));
 #  ifdef PIPEPROCS
 	if (started)
 		kbd_strt();
@@ -542,7 +533,7 @@ kbd_getch()
 #   ifdef UNIX
 					InSlowRead = NO;
 #   endif
-				} while (nchars < 0 && errno == EINTR);
+				} while (nchars < 0 && RETRY_ERRNO(errno));
 				if (nchars <= 0)
 					finish(SIGHUP);
 			}
@@ -653,7 +644,7 @@ int	delay;
 	start = Ticks;
 
 	end = start + delay * 6;
-	do ; while (!charp() && Ticks < end);
+	do {} while (!charp() && Ticks < end);
 
 #else /* !MAC */
 
@@ -1199,6 +1190,7 @@ char	*argv[];
 		case '-':
 			switch (argv[1][1]) {
 			case 'd':	/* Ignore current directory path */
+			case 'D':	/* Ignore debug file name */
 			case 'l':	/* Ignore libdir path */
 			case 's':	/* Ignore sharedir path */
 				argv += 1;
@@ -1359,16 +1351,16 @@ raw_complain(fmt, va_alist)
 {
 	char	buf[MESG_SIZE];
 	va_list	ap;
-	UnivConstPtr	bp;
+	const char *bp;
 	size_t	rem;
-	ssize_t r;
+	SSIZE_T r;
 
 	va_init(ap, fmt);
 	format(buf, sizeof(buf) - 2, fmt, ap);
 	va_end(ap);
 	strcat(buf, "\r\n");	/* \r *may* be redundant */
 	for (bp = buf, rem = strlen(buf)
-	; rem > 0 && (r = write(2, bp, rem)) != rem
+	; rem > 0 && (r = write(2, bp, rem)) != (SSIZE_T)rem
 	; bp += r, rem -=r) {
 		if (r < 0 || errno != EINTR)
 		    break;	/* give up */
@@ -1618,7 +1610,7 @@ char	*argv[];
 	argc = getArgs(&argv);
 #endif
 
-#if defined(__WATCOMC__) && defined(FAR_LINES)
+#ifdef OWCDOS
 	/* Watcom C under DOS won't grow the near heap after any far
 	 * allocation, so we must bump it up to the full 64K now.
 	 */
@@ -1681,6 +1673,12 @@ char	*argv[];
 	else
 		getCWD();	/* After we setup curbuf in case we have to getwd() */
 
+	if ((argp = scanvec(argv, "-D")) != NULL) {
+		jdpath = argp[1];
+		jdbg("jove debug started %s\n",
+		     get_time((time_t *)NULL, (char *)NULL, 0, -1));
+	}
+
 #ifdef MAC
 	HomeDir = gethome();
 #else /* !MAC */
@@ -1768,7 +1766,7 @@ char	*argv[];
 	{
 		char	*cp = &TmpDir[strlen(TmpDir)];
 
-		do ; while (cp != TmpDir && (*--cp == '/'
+		do {} while (cp != TmpDir && (*--cp == '/'
 #ifdef MSFILESYSTEM
 			|| *cp == '\\'
 #endif
@@ -1796,16 +1794,28 @@ char	*argv[];
 #endif
 
 #ifdef UNIX
-# ifndef DEBUGCRASH
-	(void) setsighandler(SIGHUP, finish);
+	/*
+	 * Jove binds INTR to a key, typically ^], which can
+	 * sometimes get hit accidentally, but it prompts in the handler.
+	 */
 	(void) setsighandler(SIGINT, handle_sigint);
+	/*
+	 * always cleanup on HUP (terminal disconnected), or
+	 * TERM (typically, system going down)
+	 */
+	(void) setsighandler(SIGHUP, finish);
+	(void) setsighandler(SIGTERM, finish);
+# ifndef DEBUGCRASH
+	/*
+	 * DEBUGCRASH means we do not set handlers for SIGBUS,
+	 * SIGSEGV, and SIGPIPE, so this can drop into a debugger or
+	 * leave a core file to assist debugging.
+	 */
 #  ifdef SIGBUS
 	(void) setsighandler(SIGBUS, finish);
 #  endif /* SIGBUS */
 	(void) setsighandler(SIGSEGV, finish);
 	(void) setsighandler(SIGPIPE, finish);
-	/* ??? Why should we ignore SIGTERM? */
-	/* (void) setsighandler(SIGTERM, SIG_IGN); */
 # endif /* DEBUGCRASH */
 	(void) setsighandler(SIGALRM, AlarmHandler);
 	SetClockAlarm(NO);
