@@ -27,7 +27,6 @@
 #include "wind.h"
 
 #include <signal.h>
-#include <errno.h>
 
 #ifdef MSDOS_PROCS
 # include <io.h>
@@ -62,6 +61,10 @@ char	ErrFmtStr[256] = "^\\{\"\\|\\}\\([^:\"( \t]*\\)\\{\"\\, line \\|:\\|(\\} *\
 \\|::  *\\([^(]*\\)(\\([0-9]*\\))$\
 \\|( \\([^(]*\\)(\\([0-9]*\\)) ),";	/* VAR: format string for parse errors */
 
+#ifdef SPELL
+char	SpellCmdFmt[FILESIZE] = SPELL;	/* VAR: command to use for spell check */
+#endif
+
 struct error {
 	Buffer		*er_buf;	/* Buffer error is in */
 	LinePtr		er_mess,	/* Actual error message */
@@ -159,11 +162,13 @@ char	*fname,
 void
 ErrFree()
 {
-	register struct error	*ep;
+	cur_error = NULL;
+	while (errorlist != NULL) {
+		struct error	*nel = errorlist->er_next;
 
-	for (ep = errorlist; ep != NULL; ep = ep->er_next)
-		free((UnivPtr) ep);
-	errorlist = cur_error = NULL;
+		free((UnivPtr) errorlist);
+		errorlist = nel;
+	}
 }
 
 /* Parse errors of the form specified in ErrFmtStr in the current
@@ -330,17 +335,27 @@ char	*command;
 	static char	bnm[FILESIZE];
 	register char	*cp = bnm,
 			c;
+	const char	*bp;
 
 	do {
 		c = *command++;
 	} while (jiswhite(c));
+	*cp++ = '['; /* avoid potential clash with script or program names */
 	while (cp < &bnm[sizeof(bnm) - 1] && c != '\0' && !jiswhite(c)) {
 		*cp++ = c;
 		c = *command++;
 	}
 	*cp = '\0';
-	strcpy(bnm, jbasename(bnm));
-
+	if ((bp = jbasename(&bnm[1])) != &bnm[1]) {
+		/* bp overlaps bnm, cannot use strcpy */
+		cp = &bnm[1];
+		do ; while ((*cp++ = *bp++) != '\0');
+		cp--; /* make sure cp points at '\0' */
+	}
+	if (cp > &bnm[1] && cp < &bnm[sizeof(bnm)-1]) {
+		*cp++ = ']';
+		*cp = '\0';
+	}
 	return bnm;
 }
 
@@ -439,14 +454,18 @@ void
 SpelBuffer()
 {
 	static const char	Spell[] = "Spell";
+	const char *cp;
 	char	com[100];
 	Buffer	*savebp = curbuf;
 
 	if (curbuf->b_fname == NULL)
 		complain("no file name");
+	if ((cp = strchr(SpellCmdFmt, '%')) == NULL ||
+	    cp[1] != 's' || strchr(cp+2, '%') != NULL)
+		complain("spell-command-format needs one %%s with no other format characters");
 	if (IsModified(curbuf))
 		SaveFile();
-	swritef(com, sizeof(com), "spell %s", curbuf->b_fname);
+	swritef(com, sizeof(com), SpellCmdFmt, curbuf->b_fname);
 	(void) UnixToBuf(UTB_DISP|UTB_CLOBBER|UTB_ERRWIN|UTB_SH,
 		Spell, (char *)NULL, com);
 	message("[Delete the irrelevant words and then type ^X ^C]");
@@ -617,6 +636,36 @@ wait_status_t	*status;	/* may be NULL */
 
 #endif /* !MSDOS_PROCS */
 
+/* Environment management for subshells */
+private Env proc_env;
+private char PEnvExpBuf[LBSIZE];
+private char PEnvUnsetBuf[LBSIZE];
+
+void
+ProcEnvExport()
+{
+	jamstr(PEnvExpBuf, ask(PEnvExpBuf, ProcFmt));
+	jputenv(&proc_env, PEnvExpBuf);
+}
+
+void
+ProcEnvShow()
+{
+	const char **p;
+	TOstart("subshell environment");
+	for (p = jenvdata(&proc_env); *p; p++) {
+	    Typeout("%s", *p);
+	}
+	TOstop();
+}
+
+void
+ProcEnvUnset()
+{
+	jamstr(PEnvUnsetBuf, ask(PEnvUnsetBuf, ProcFmt));
+	junsetenv(&proc_env, PEnvUnsetBuf);
+}
+
 /* Run the command cmd.  Output to the buffer named bnm (if not
  * NULL), first erasing bnm (if UTB_DISP and UTB_CLOBBER), and
  * redisplay (if UTB_DISP).  Leaves bnm as the current buffer and
@@ -667,6 +716,7 @@ UnixToBuf(flags, bnm, InFName, cmd)
 	SIGINTMASK_INIT();
 #endif
 
+	jdbg("UnixToBuf flags 0x%x \"%s\"\n", flags, cmd);
 	SlowCmd += 1;;
 	if (flags & UTB_SH) {
 			*ap++ = Shell;
@@ -729,7 +779,7 @@ UnixToBuf(flags, bnm, InFName, cmd)
 	}
 	*ap++ = NULL;
 
-	if (access(argv[0], X_OK) != 0) {
+	if (access(argv[0], J_X_OK) != 0) {
 		complain("[Couldn't access %s: %s]", argv[0], strerror(errno));
 		/* NOTREACHED */
 	}
@@ -778,6 +828,18 @@ UnixToBuf(flags, bnm, InFName, cmd)
 # endif
 
 # ifdef USE_VFORK
+	/*
+	 * There are several other forks in Jove, but for
+	 * machines lacking MMUs, where vfork might be
+	 * significantly more performant, it is nice to have
+	 * SUBSHELL capability using pipes (MSDOS_PROCS use a
+	 * temporary file for subshell output and spawn).
+	 * moraes Note from 2020 4.17: vfork has not been tested in a
+	 * long time, it is quite possible/likely that it does
+	 * not even work (I think that nothing between here
+	 * and the exec damages parent state, but using vfork
+	 * is a deal with the devil)
+	 */
 	ChildPid = vfork();
 # else
 	ChildPid = fork();
@@ -795,31 +857,24 @@ UnixToBuf(flags, bnm, InFName, cmd)
 		complain("[Fork failed: %s]", strerror(fork_errno));
 	}
 	if (ChildPid == 0) {
-# ifdef USE_VFORK
-		/* There are several other forks in Jove, but this is
-		 * the only one we execute often enough to make it worth
-		 * using a vfork.  This assumes a system with vfork also
-		 * has BSD signals!
-		 */
+		const char	*a;	/* action name (for error message) */
 		(void) setsighandler(SIGINT, SIG_DFL);
 #  ifdef SIGINT_UNBLOCK
 		SIGINT_UNBLOCK();
 #  endif
-# else /* !USE_VFORK */
-		(void) setsighandler(SIGINT, SIG_DFL);
-#  ifdef SIGINT_UNBLOCK
-		SIGINT_UNBLOCK();
-#  endif
-# endif /* !USE_VFORK */
-		(void) close(0);
-		(void) open(InFName==NULL? "/dev/null" : InFName, O_RDONLY | O_BINARY);
-		(void) close(1);
-		(void) dup(p[1]);
-		(void) close(2);
-		(void) dup(p[1]);
+		if (!((a = "close 0", close(0)) == 0
+		&& (a = "open", open(InFName==NULL? "/dev/null" : InFName, O_RDONLY | O_BINARY)) == 0
+		&& (a = "close 1", close(1)) == 0
+		&& (a = "dup 1", dup(p[1])) == 1
+		&& (a = "close 2", close(2)) == 0
+		&& (a = "dup 2", dup(p[1])) == 2)) {
+			raw_complain("% in setup for child failed: %s", a, strerror(errno));
+			_exit(1);
+		}
+
 		pipeclose(p);
 		jcloseall();
-		execv(argv[0], &argv[1]);
+		execve(argv[0], &argv[1], (char **) jenvdata(&proc_env));
 		raw_complain("Execl failed: %s", strerror(errno));
 		_exit(1);
 	}
@@ -833,37 +888,51 @@ UnixToBuf(flags, bnm, InFName, cmd)
 		int	oldi = dup(0),
 			oldo = dup(1),
 			olde = dup(2);
-		bool	InFailure;
+		bool	InFailure = NO;
 		int	ph;
 
 		PathCat(pipename, sizeof(pipename), TmpDir, "jpXXXXXX");
 		ph = MakeTemp(pipename, "cannot make tempfile \"%s\" for filter");
-		close(1);
-		close(2);
-		dup(ph);
-		dup(ph);
-		close(ph);
+		jdbg("created temp \"%s\"\n", pipename);
+
+		if (dup2(ph, 1) < 0 || dup2(ph, 2) < 0) {
+			jdbg("dup2 of temp file fd failed: %d %s\n", errno,
+			     strerror(errno));
+			InFailure = YES;
+		}
+		(void) close(ph);
+		(void) close(0);
 
-		close(0);
-		InFailure = InFName != NULL && open(InFName, O_RDONLY | O_BINARY) < 0;
 		if (!InFailure)
-			status = spawnv(0, argv[0], &argv[1]);
-
-		close(0);
-		close(1);
-		close(2);
-		dup(oldi);
-		dup(oldo);
-		dup(olde);
-		close(oldi);
-		close(oldo);
-		close(olde);
+			InFailure = InFName != NULL && open(InFName, O_RDONLY | O_BINARY) < 0;
+		jdbg("InFailure %d InFName \"%s\"\n", InFailure,
+		     InFName == NULL ? "(NULL)" : InFName);
+		if (!InFailure) {
+			if (jdebug) {
+				int vk = 0;
+				while (argv[vk]) {
+					jdprintf("%d: \"%s\"\n", vk, argv[vk]);
+					vk++;
+				}
+			}
+			status = spawnve(0, argv[0], &argv[1],
+					 jenvdata(&proc_env));
+			jdbg("status %d spawnve \"%s\"\n", status, argv[0]);
+		}
+		jdbg("restoring fds 0 1 2 from %d %d %d\n", oldi, oldo, olde);
+		(void) dup2(oldi, 0);
+		(void) dup2(oldo, 1);
+		(void) dup2(olde, 2);
+		(void) close(oldi);
+		(void) close(oldo);
+		(void) close(olde);
 
 		if (InFailure)
 			complain("[filter input failed]");
 		if (status < 0)
 			s_mess("[Spawn failed %d]", errno);
-		ph = open(pipename, O_RDONLY | O_BINARY);
+		jdbg("opening \"%s\" from \"%s\"\n", pipename, argv[1]);
+		ph = open(pipename, O_RDONLY | O_BINARY | O_CLOEXEC);
 		if (ph < 0)
 			complain("[cannot reopen pipe]", strerror(errno));
 		fp = fd_open(argv[1], F_READ, ph, iobuff, LBSIZE);
