@@ -18,9 +18,7 @@
  * Look in Makefile to change the default directories.
  */
 
-#include <errno.h>
-#include <stdio.h>	/* Do stdio first so it doesn't override OUR
-			   definitions. */
+#define USE_STDIO_H 1
 #include "jove.h"
 
 #define SMALLSTRSIZE	30	/* used for small buffers */
@@ -166,7 +164,7 @@ size_t	n;
 private char	*getblock proto((daddr atl));
 
 void
-getline(tl, buf)
+jgetline(tl, buf)
 daddr	tl;
 char	*buf;
 {
@@ -201,7 +199,7 @@ daddr	atl;
 
 	if (bno != curblock) {
 		ssize_t r;
-		char *what;
+		const char *what;
 
 		errno = 0;
 		what = "lseek";
@@ -223,6 +221,7 @@ daddr	atl;
 	return blk_buf + off;
 }
 
+/* This pre-dates strdup, but can we assume it exists now? */
 char *
 copystr(s)
 const char	*s;
@@ -247,6 +246,18 @@ private const char	*CurDir;
 
 private bool	add_name proto((char *));
 
+private void
+free_files() {
+	while (First != NULL) {
+		struct file_pair	*p = First;
+
+		First = p->file_next;
+		free((UnivPtr) p->file_data);
+		free((UnivPtr) p->file_rec);
+		free((UnivPtr) p);
+	}
+}
+
 private void
 get_files(dirname)
 const char	*dirname;
@@ -255,12 +266,7 @@ const char	*dirname;
 	int	nentries;
 
 	/* first, free any previous entries */
-	while (First != NULL) {
-		struct file_pair	*p = First;
-
-		First = p->file_next;
-		free((UnivPtr) p);
-	}
+	free_files();
 
 	CurDir = dirname;
 	nentries = jscandir(dirname, &nmptr, add_name,
@@ -273,8 +279,8 @@ private bool
 add_name(fname)
 char *fname;
 {
-	char	dfile[FILESIZE],
-		rfile[FILESIZE];
+	char	dfile[FILESIZE*2+2], /* CurDir/filename */
+		rfile[FILESIZE*2+2];
 	struct file_pair	*fp;
 	struct rec_head		header;
 	int	fd;
@@ -290,7 +296,7 @@ char *fname;
 	 * in saving its name.
 	 */
 	(void) sprintf(rfile, "%s/%s", CurDir, fname);
-	if ((fd = open(rfile, O_RDONLY | O_BINARY)) != -1) {
+	if ((fd = open(rfile, O_RDONLY | O_BINARY | O_CLOEXEC)) != -1) {
 		if (read(fd, (UnivPtr) &header, sizeof header) != sizeof header) {
 			close(fd);
 			fprintf(stderr, "recover: could not read complete header from %s, skipping\n", rfile);
@@ -400,7 +406,7 @@ size_t	buflen;
 	char	*bp = buf,
 		*ep = buf + buflen - 1;
 
-	do ; while (strchr(" \t\n", c = getchar()) != NULL);
+	do {} while (strchr(" \t\n", c = getchar()) != NULL);
 
 	for (;;) {
 		if (c == EOF)
@@ -449,7 +455,7 @@ private void
 restore()
 {
 	register int	i;
-	char	tofile[FILESIZE],
+	char	tofile[FILESIZE+1], /* leading # */
 		answer[SMALLSTRSIZE];
 	int	nrecovered = 0;
 
@@ -489,9 +495,9 @@ char	*dest;
 	if (src == NULL || dest == NULL)
 		return;
 
-	if (dest == tty)
+	if (dest == tty) {
 		outfile = stdout;
-	else {
+	} else {
 		if ((outfile = fopen(dest, "wb")) == NULL) {
 			printf("recover: cannot create %s.\n", dest);
 			(void) signal(SIGINT, SIG_DFL);
@@ -501,7 +507,7 @@ char	*dest;
 	}
 	if (setjmp(int_env) == 0) {
 		(void) signal(SIGINT, catch);
-		dump_file(src - buflist, outfile);
+		dump_file((int)(src - buflist), outfile);
 	} else {
 		printf("\nAborted!\n");
 	}
@@ -597,7 +603,7 @@ FILE	*out;
 	Nchars = Nlines = 0L;
 	while (--nlines >= 0) {
 		addr = getaddr(ptrs_fp);
-		getline(addr, buf);
+		jgetline(addr, buf);
 		Nlines += 1;
 		Nchars += 1 + strlen(buf);
 		fputs(buf, out);
@@ -663,7 +669,7 @@ struct file_pair	*fp;
 		Header.Nbuffers,
 		Header.Nbuffers != 1 ? "s" : "",
 		ctime(&Header.UpdTime));
-	data_fd = open(datafile, O_RDONLY | O_BINARY);
+	data_fd = open(datafile, O_RDONLY | O_BINARY | O_CLOEXEC);
 	if (data_fd == -1) {
 		fprintf(stderr, "recover: but I can't read the data file (%s).\n", datafile);
 		ask_del("Should I delete the tmp files? ", fp);
@@ -770,6 +776,7 @@ struct rec_head *rec;
 	const char *buf_string;
 	FILE *mail_pipe;
 	struct passwd *pw;
+	int r;
 
 	if ((pw = getpwuid(rec->Uid))== NULL)
 		return;
@@ -777,7 +784,13 @@ struct rec_head *rec;
 	last_update = ctime(&(rec->UpdTime));
 	/* Start up mail */
 	sprintf(mail_cmd, "/bin/mail %s", pw->pw_name);
-	setuid(getuid());
+	if ((r = setuid(getuid())) < 0) {
+	    perror("WARNING: jove recover: setuid(getuid()) failed");
+	    /*
+	     * used to continue without checking return value,
+	     * so let that behaviour continue, I guess?
+	     */
+	}
 	if ((mail_pipe = popen(mail_cmd, "w")) == NULL)
 		return;
 
@@ -829,7 +842,7 @@ savetmps()
 		case 0:
 			fprintf(stderr, "Recovering: %s, %s\n", fp->file_data,
 			 fp->file_rec);
-			if ((fd = open(fp->file_rec, O_RDONLY | O_BINARY)) != -1) {
+			if ((fd = open(fp->file_rec, O_RDONLY | O_BINARY | O_CLOEXEC)) != -1) {
 				if ((read(fd, (UnivPtr) &header, sizeof header) != sizeof header)) {
 					close(fd);
 					return;
@@ -844,7 +857,7 @@ savetmps()
 			/*NOTREACHED*/
 
 		default:
-			do ; while (wait(&status) != pid);
+			do {} while (wait(&status) != pid);
 			if (WIFSIGNALED(status))
 				fprintf(stderr, "recover: copy terminated by signal %d\n.\n", WTERMSIG(status));
 			if (WIFEXITED(status))
@@ -861,6 +874,7 @@ savetmps()
 				perror("recover: chown failed.");
 		}
 	}
+	free_files();
 }
 
 private int
