@@ -5,7 +5,7 @@
  * this notice is included in all the source files and documentation.     *
  **************************************************************************/
 
-#include <stdio.h>
+#define USE_STDIO_H 1
 #include "jove.h"
 #include "chars.h"
 #include "commands.h"
@@ -14,7 +14,7 @@
 #define LINESIZE	100	/* hope this is big enough */
 #define STACKLIMIT	10	/* max conditional depth */
 
-#define	PROC(p)	NULL	/* discard function pointers */
+#define	PROC(p)	NULLPROC	/* discard function pointers */
 #include "commands.tab"
 
 #define VAR(v)	NULL, (size_t)0	/* discard variable pointers */
@@ -72,14 +72,18 @@ int	c;
 		strcpy(cp, "M-");
 		cp += 2;
 	}
-	if (c == ESC)
+	if (c == ESC) {
 		strcpy(cp, "ESC");
-	else if (c < ' ')
-		(void) sprintf(cp, "^%c", c + '@');
-	else if (c == DEL)
+	} else if (c < ' ') {
+		*cp++ = '^';
+		*cp++ = c + '@';
+		*cp = '\0';
+	} else if (c == DEL) {
 		strcpy(cp, "^?");
-	else
-		(void) sprintf(cp, "%c", c);
+	} else {
+		*cp++ = c;
+		*cp = '\0';
+	}
 	return str;
 }
 
@@ -168,7 +172,7 @@ for (fnp = fnt; fnp->in != NULL; fnp++) {
 	lino = 0;
 	ch = 0;
 	for (;;) {
-		if (fgets(line, sizeof line, ifile) == NULL) {
+		if (fgets(line, (int)sizeof(line), ifile) == NULL) {
 			if (sp != stackspace) {
 				fprintf(stderr, "EOF inside #if\n");
 				exit(1);
