@@ -6,7 +6,6 @@
  **************************************************************************/
 
 #include "jove.h"
-#include <errno.h>
 #include "jctype.h"
 #include "disp.h"
 #include "fp.h"
@@ -120,16 +119,20 @@ int	char1,
 		if (prevp == endp)
 			return NO;
 	}
-	while (nextp!=NULL && nextp!=endp) {
+	while (nextp!=NULL) {
 		nextp = nextp->l_next;
 		line_diff += 1;
+		if (nextp == endp)
+			return YES;
 	}
-	while (prevp!=NULL && prevp!=endp) {
+	while (prevp!=NULL) {
 		prevp = prevp->l_prev;
 		line_diff += 1;
+		if (prevp == endp)
+			return NO;
 	}
-	/* nextp == prevp implies both are NULL: the lines are not ordered */
-	return nextp==prevp? -1 : nextp==endp;
+	/* the lines are not ordered */
+	return -1;
 }
 
 /* Number of lines (forward OR back) from nextp to endp.
@@ -248,7 +251,7 @@ register const Buffer	*b;
 }
 
 int
-min(a, b)
+jmin(a, b)
 register int	a,
 		b;
 {
@@ -256,7 +259,7 @@ register int	a,
 }
 
 int
-max(a, b)
+jmax(a, b)
 register int	a,
 		b;
 {
@@ -279,8 +282,9 @@ char	*buf;
 		if (buf != linebuf)
 			strcpy(buf, linebuf);
 		Jr_Len = strlen(linebuf);
-	} else
-		getline(line->l_dline, buf);
+	} else {
+		jgetline(line->l_dline, buf);
+	}
 	return buf;
 }
 
@@ -435,7 +439,7 @@ int	flag;
 
 void
 ins_c(c, buf, atchar, num, max)
-char	c;
+DAPchar	c;
 char	*buf;
 int	atchar,
 	num,
@@ -687,13 +691,16 @@ const char *str;
 void
 jamstrsub(buf, str, bufsz)
 char *buf;
-size_t bufsz;
 const char *str;
+size_t bufsz;
 {
-	if (strlen(str) < bufsz)
-		strcpy(buf, str);
-	else
+	size_t strsz = strlen(str);
+	if (strsz < bufsz) {
+		byte_move(str, buf, strsz);
+		buf[strsz] = '\0';
+	} else {
 		complain("string too long");
+	}
 }
 
 bool
@@ -778,3 +785,93 @@ ZXchar	first, second;
 	}
 	return second;
 }
+
+#if defined(IPROCS) || defined(SUBSHELL)
+/* Convenience routine, initializes envp properly if it is null,
+ * hides a bit of the abstraction and some duplicated code
+ */
+const char **
+jenvdata(envp)
+Env *envp;
+{
+	if (envp->e_data == NULL) {
+	    envp->e_data = (const char **) environ; /* avoid gcc warning */
+	    envp->e_malloced = NO;
+	    envp->e_headroom = 0;
+	}
+	return envp->e_data;
+}
+    
+/* Put a definition into the environment.
+ * Same as putenv(3) in SVID 3, POSIX, and BSD 4.3.
+ */
+void
+jputenv(envp, def)
+Env *envp;
+const char *def;
+{
+	const char **p, **e;
+	const char *eq;
+
+	if ((eq = strchr(def, '=')) == NULL)
+		return;
+	for (p = e = jenvdata(envp); ; p++) {
+		if (*p == NULL) {
+			if (envp->e_headroom == 0) {
+#				define JENV_INCR	5
+				size_t	sz = ((p-e) + 1) * sizeof(char *);
+				const char	**ne = (const char **)
+					malloc(sz + JENV_INCR*sizeof(char *));
+
+				if (ne == NULL)
+					break;	/* malloc failed: give up -- doesn't matter much */
+
+				byte_copy(envp->e_data, ne, sz);
+				p = ne + (p-e);
+				if (envp->e_malloced)
+					free((UnivPtr)envp->e_data);
+				envp->e_data = e = ne;
+				envp->e_malloced = YES;
+				envp->e_headroom = JENV_INCR;
+#				undef JENV_INCR
+			}
+			envp->e_headroom -= 1;
+			*p++ = copystr(def);
+			*p = NULL;
+			break;
+		}
+		if (strncmp(*p, def, (size_t) (eq - def + 1)) == 0) {
+			*p = copystr(def);
+			break;
+		}
+	}
+}
+
+/* Remove any definitions of name from the environment.
+ * Same as 4.3BSD's unsetenv(3).
+ */
+void
+junsetenv(envp, name)
+Env *envp;
+const char *name;
+{
+	const char **p, **q;
+	size_t l = strlen(name);
+
+	for (p = q = jenvdata(envp);;) {
+		const char *e = *p++;
+
+		*q = e;
+
+		if (e == NULL)
+			break;
+
+		if (strncmp(e, name, l) == 0 && e[l] == '=') {
+			/* unset this one by not advancing q */
+			envp->e_headroom += 1;
+		} else {
+			q += 1;
+		}
+	}
+}
+#endif /* defined(IPROCS) */
