@@ -11,14 +11,14 @@ extern int
 	fnamecomp proto((UnivConstPtr, UnivConstPtr));	/* order file names */
 
 #define IsModified(b)	((b)->b_modified)
-#define SavLine(a, b)	((a)->l_dline = putline((b)))
+#define SavLine(a, b)	((a)->l_dline = jputline((b)))
 #define SetLine(line)	DotTo((line), 0)
 #define bobp()		(firstp(curline) && bolp())
 #define bolp()		(curchar == 0)
 #define eobp()		(lastp(curline) && eolp())
 #define eolp()		(linebuf[curchar] == '\0')
 #define firstp(line)	((line) == curbuf->b_first)
-#define getDOT()	getline(curline->l_dline, linebuf)
+#define getDOT()	jgetline(curline->l_dline, linebuf)
 #define lastp(line)	((line) == curbuf->b_last)
 
 extern UnivPtr
@@ -41,8 +41,8 @@ extern int
 	LineDist proto((LinePtr nextp,LinePtr endp)),
 	LinesTo proto((LinePtr from, LinePtr to)),
 	length proto((LinePtr line)),
-	max proto((int a,int b)),
-	min proto((int a,int b)),
+	jmax proto((int a,int b)),
+	jmin proto((int a,int b)),
 	numcomp proto((const char *s1, const char *s2)),
 	numcompcase proto((const char *s1, const char *s2));
 
@@ -84,6 +84,35 @@ extern void
 extern UnivPtr
 	freealloc proto((UnivPtr obj, size_t size));
 
+#if defined(IPROCS) || defined(SUBSHELL)
+
+# ifndef OWCDOS
+/*
+ * The Open Watcom C declaration of environ has some macro
+ * decorations that do not match with this, so we skip this,
+ * their declaration works and is available.
+ * Linux only provides this in unistd.h if _GNU_SOURCE is defined,
+ * which we do not.
+ * *BSD and OpenIndiana (Solaris) and probably most other
+ * systems do not provide it, since it is not part of POSIX,
+ * but environ seems remarkably portable, nonetheless!
+ */
+extern char **environ;	/* <unistd.h> */
+# endif
+
+typedef struct {
+	const char **e_data;
+	bool e_malloced;
+	int e_headroom;	/* remaining room in e_data array */
+} Env;
+
+extern const char **
+	jenvdata proto((Env *envp));
+extern void
+	jputenv proto((Env *envp, const char *def)),
+	junsetenv proto((Env *envp, const char *name));
+#endif /* defined(IPROCS) */
+
 /* copy a string into a possibly-too-small buffer
  * trunc* silently truncates a too-large string.
  * jam* complain if the string is too large.
@@ -91,8 +120,8 @@ extern UnivPtr
 #define truncstr(buf, str)	truncstrsub(buf, str, sizeof(buf))
 #define jamstr(buf, str)	jamstrsub(buf, str, sizeof(buf))
 extern void
-	truncstrsub proto((unsigned char *buf, const unsigned char *str, size_t bufsz)),
-	jamstrsub proto((unsigned char *buf, const unsigned char *str, size_t bufsz));
+	truncstrsub proto((char *buf, const char *str, size_t bufsz)),
+	jamstrsub proto((char *buf, const char *str, size_t bufsz));
 
 /* Variables: */
 
